@@ -22,6 +22,7 @@ use Socket ();
 use File::Spec::Functions qw(catfile);
 
 use Apache::TestTrace;
+use Apache::TestRun;
 use Apache::TestConfig ();
 use Apache::TestRequest ();
 
@@ -52,16 +53,51 @@ sub new {
 
     $self->{port_counter} = $self->{config}->{vars}->{port};
 
+    $self;
+}
+
+# call this when you already know where httpd is
+sub post_config {
+    my($self) = @_;
+
     $self->{version} = $self->{config}->httpd_version || '';
     $self->{mpm}     = $self->{config}->httpd_mpm     || '';
-    ($self->{rev})   = $self->{version} =~ m:^Apache/(\d)\.:;
-    $self->{rev}   ||= 2;
+
+    # try to get the revision number from the standard Apache version
+    # string and various variations made by distributions which mangle
+    # that string
+
+    # Apache/2.0.50-dev
+    ($self->{rev})   = $self->{version} =~ m|^Apache/(\d)\.|;
+
+    # Apache-AdvancedExtranetServer/1.3.29 (Mandrake Linux/1mdk)
+    ($self->{rev}) ||= $self->{version} =~ m|^Apache.*?/(\d)\.|;
+
+    # IBM_HTTP_SERVER/1.3.19  Apache/1.3.20 (Unix)
+    ($self->{rev}) ||= $self->{version} =~ m|^.*?Apache.*?/(\d)\.|;
+
+    if ($self->{rev}) {
+        debug "Matched Apache revision $self->{version} $self->{rev}";
+    }
+    else {
+        # guessing is not good as it'll only mislead users
+        # and we can't die since a config object is required
+        # during Makefile.PL's write_perlscript when path to httpd may
+        # be unknown yet. so default to non-existing version 0 for now.
+        # and let TestRun.pm figure out the required pieces
+        debug "can't figure out Apache revision, from string: " .
+            "'$self->{version}', using a non-existing revision 0";
+        $self->{rev} = 0; # unknown
+    }
 
     $self;
 }
 
 sub version_of {
     my($self, $thing) = @_;
+    die "Can't figure out what Apache server generation we are running"
+        unless $self->{rev};
+
     $thing->{$self->{rev}};
 }
 
@@ -123,10 +159,20 @@ sub args {
 my %one_process = (1 => '-X', 2 => '-D ONE_PROCESS');
 
 sub start_cmd {
-    my $self = shift;
+    my $self  = shift;
+
+    my $args   = $self->args;
+    my $config = $self->{config};
+    my $vars   = $config->{vars};
+    my $httpd  = $vars->{httpd};
+
+    my $one_process = $self->{run}->{opts}->{'one-process'}
+        ? $self->version_of(\%one_process)
+        : '';
+
     #XXX: threaded mpm does not respond to SIGTERM with -D ONE_PROCESS
-    my $args = $self->args;
-    return "$self->{config}->{vars}->{httpd} $args";
+
+    return "$httpd $one_process $args";
 }
 
 sub default_gdbinit {
